@@ -8,6 +8,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
@@ -414,6 +415,19 @@ public class LiferayGroupManager implements GroupManager {
 		}
 		return toReturn;
 	}
+
+	@Override
+	public List<GCubeGroup> listVresByUser(long userId)
+			throws UserManagementSystemException, GroupRetrievalFault, UserRetrievalFault {
+		List<GCubeGroup> toReturn = listGroupsByUser(userId);
+		Iterator<GCubeGroup> iterator = toReturn.iterator();
+		while (iterator.hasNext()) {
+			GCubeGroup gCubeGroup = (GCubeGroup) iterator.next();
+			if(!isVRE(gCubeGroup.getGroupId()))
+				iterator.remove();
+		}
+		return toReturn;
+	}
 	/**
 	 * {@inheritDoc}
 	 */
@@ -730,6 +744,4 @@ public class LiferayGroupManager implements GroupManager {
 		}
 		return gateways;
 	}
-
-
-}
+}
\ No newline at end of file
