@@ -6,10 +6,12 @@ import java.net.URLEncoder;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.util.ArrayList;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Executors;
+import java.util.concurrent.TimeUnit;
 
 import org.apache.commons.codec.binary.Base64;
 import org.apache.http.HttpHost;
@@ -17,15 +19,10 @@ import org.apache.http.auth.AuthScope;
 import org.apache.http.auth.UsernamePasswordCredentials;
 import org.apache.http.client.AuthCache;
 import org.apache.http.client.CredentialsProvider;
-import org.apache.http.client.methods.CloseableHttpResponse;
-import org.apache.http.client.methods.HttpGet;
 import org.apache.http.client.protocol.HttpClientContext;
 import org.apache.http.impl.auth.BasicScheme;
 import org.apache.http.impl.client.BasicAuthCache;
 import org.apache.http.impl.client.BasicCredentialsProvider;
-import org.apache.http.impl.client.CloseableHttpClient;
-import org.apache.http.impl.client.HttpClients;
-import org.apache.http.util.EntityUtils;
 import org.gcube.vomanagement.usermanagement.UserManager;
 import org.gcube.vomanagement.usermanagement.exception.GroupRetrievalFault;
 import org.gcube.vomanagement.usermanagement.exception.RoleRetrievalFault;
@@ -33,6 +30,7 @@ import org.gcube.vomanagement.usermanagement.exception.TeamRetrievalFault;
 import org.gcube.vomanagement.usermanagement.exception.UserManagementPortalException;
 import org.gcube.vomanagement.usermanagement.exception.UserManagementSystemException;
 import org.gcube.vomanagement.usermanagement.exception.UserRetrievalFault;
+import org.gcube.vomanagement.usermanagement.impl.ws.utils.HttpUtils;
 import org.gcube.vomanagement.usermanagement.model.Email;
 import org.gcube.vomanagement.usermanagement.model.GCubeMembershipRequest;
 import org.gcube.vomanagement.usermanagement.model.GCubeRole;
@@ -87,6 +85,7 @@ public class LiferayWSUserManager implements UserManager{
 
 	// some pre-defined constants
 	private static final String USER_LOCATION_INDUSTRY_KEY = "industry";
+	private static final int USERS_EXECUTOR_FACTOR = 20;
 
 	/**
 	 * In order to contact the json ws of Liferay, user and password are needed. The host in which the current JVM
@@ -128,7 +127,7 @@ public class LiferayWSUserManager implements UserManager{
 	 */
 	private void retrieveCompanyId() throws Exception {
 
-		String json = executeHTTPGETRequest(API_BASE_URL + GET_COMPANY_ID, credsProvider, localContext, target);
+		String json = HttpUtils.executeHTTPGETRequest(API_BASE_URL + GET_COMPANY_ID, credsProvider, localContext, target);
 
 		if(json != null){
 			JSONParser parser = new JSONParser();
@@ -140,37 +139,6 @@ public class LiferayWSUserManager implements UserManager{
 
 	}
 
-	/**
-	 * Execute an http GET request and returns the JSON response
-	 * @param requestPath
-	 * @return a JSON string on success, null otherwise
-	 */
-	private static String executeHTTPGETRequest(String requestPath, CredentialsProvider credsProvider, HttpClientContext localContext, HttpHost target){
-
-		try{
-			CloseableHttpClient httpclient = HttpClients.custom()
-					.setDefaultCredentialsProvider(credsProvider).build();
-
-			HttpGet httpget = new HttpGet(requestPath);
-
-			logger.debug("Executing request " + httpget.getRequestLine() + " to target " + target);
-			CloseableHttpResponse response = httpclient.execute(target, httpget, localContext);
-			try {
-				String result = EntityUtils.toString(response.getEntity());
-				logger.debug("Request result is " + result);
-				return result;
-			} finally {
-				response.close();
-			}
-		}catch(Exception e){
-			logger.error("Exception while performing GET request with path " + requestPath, e);
-		}
-
-		return null;
-
-	}
-
-
 	/**
 	 * Maps the JSON user to the GCubeUser.class object
 	 * @param json
@@ -260,13 +228,13 @@ public class LiferayWSUserManager implements UserManager{
 	}
 
 	/**
-	 * Given the contactId value, retrieves the json object releted to this information
+	 * Given the contactId value, retrieves the json object related to this information
 	 * @param contactId
 	 * @return
 	 */
 	private String getContactJson(long contactId) {
 
-		return executeHTTPGETRequest(API_BASE_URL + GET_CONTACT_BY_USER_ID.replace("$CONTACT_ID", String.valueOf(contactId)), 
+		return HttpUtils.executeHTTPGETRequest(API_BASE_URL + GET_CONTACT_BY_USER_ID.replace("$CONTACT_ID", String.valueOf(contactId)), 
 				credsProvider, localContext, target);
 	}
 
@@ -321,7 +289,7 @@ public class LiferayWSUserManager implements UserManager{
 			throws UserManagementSystemException, UserRetrievalFault {
 
 		String jsonUser = 
-				executeHTTPGETRequest(API_BASE_URL + GET_USER_BY_USERNAME.replace("$COMPANY_ID", String.valueOf(companyId)).replace("$USER_ID", username), 
+				HttpUtils.executeHTTPGETRequest(API_BASE_URL + GET_USER_BY_USERNAME.replace("$COMPANY_ID", String.valueOf(companyId)).replace("$USER_ID", username), 
 						credsProvider, localContext, target);
 
 		if(jsonUser != null){
@@ -342,7 +310,7 @@ public class LiferayWSUserManager implements UserManager{
 	public GCubeUser getUserByEmail(String email)
 			throws UserManagementSystemException, UserRetrievalFault {
 		String jsonUser = 
-				executeHTTPGETRequest(API_BASE_URL + GET_USER_BY_EMAIL.replace("$COMPANY_ID", String.valueOf(companyId)).replace("$EMAIL", email), 
+				HttpUtils.executeHTTPGETRequest(API_BASE_URL + GET_USER_BY_EMAIL.replace("$COMPANY_ID", String.valueOf(companyId)).replace("$EMAIL", email), 
 						credsProvider, localContext, target);
 
 		if(jsonUser != null){
@@ -357,7 +325,7 @@ public class LiferayWSUserManager implements UserManager{
 			throws UserManagementSystemException, UserRetrievalFault {
 
 		String jsonUser = 
-				executeHTTPGETRequest(API_BASE_URL + GET_USER_BY_ID.replace("$USER_ID", String.valueOf(userId)), 
+				HttpUtils.executeHTTPGETRequest(API_BASE_URL + GET_USER_BY_ID.replace("$USER_ID", String.valueOf(userId)), 
 						credsProvider, localContext, target);
 
 		if(jsonUser != null){
@@ -403,23 +371,60 @@ public class LiferayWSUserManager implements UserManager{
 	}
 
 	@Override
+	@SuppressWarnings("rawtypes")
 	public List<GCubeUser> listUsersByGroup(long groupId)
 			throws UserManagementSystemException, GroupRetrievalFault,
 			UserRetrievalFault {
 
 		try{
-			List<GCubeUser> toReturn = new ArrayList<GCubeUser>();
+			final List<GCubeUser> toReturn = new ArrayList<GCubeUser>();
 			String jsonUsers = 
-					executeHTTPGETRequest(API_BASE_URL + GET_USERS_BY_GROUP.replace("$GROUP_ID", String.valueOf(groupId)), 
+					HttpUtils.executeHTTPGETRequest(API_BASE_URL + GET_USERS_BY_GROUP.replace("$GROUP_ID", String.valueOf(groupId)), 
 							credsProvider, localContext, target);
 
 			if(jsonUsers != null){
+
 				logger.debug("Trying to parse json users array ");
 				JSONParser parser = new JSONParser();
 				JSONArray array = (JSONArray)parser.parse(jsonUsers);
-				for (int i = 0; i < array.size(); i++) {
-					toReturn.add(mapLRUser(((JSONObject)array.get(i)).toJSONString()));
+
+				// use executors to speed up this process
+				int numberOfThreads = (int) Math.ceil((double)array.size()/(double)USERS_EXECUTOR_FACTOR);
+				logger.trace("Number of concurrent threads is going to be " + numberOfThreads);
+
+				// let do the job by some threads
+				ExecutorService executor = Executors.newFixedThreadPool(numberOfThreads);
+				int start = 0;
+				int offset = USERS_EXECUTOR_FACTOR;
+
+				for(int i = 0; i < numberOfThreads; i++){
+					start = USERS_EXECUTOR_FACTOR * i;
+					offset = (start + offset) > array.size() ? array.size() - start: offset;
+					logger.trace("Start = " + start + ", offset=" + offset);
+					final List usersPortion = array.subList(start, start + offset);
+
+					executor.submit(new Runnable() {
+						@Override
+						public void run() {
+							logger.debug("Thread is " + Thread.currentThread().getName());
+
+							ArrayList<GCubeUser> localList = new ArrayList<GCubeUser>();
+							for(int i = 0; i < usersPortion.size(); i++){
+								localList.add(mapLRUser(((JSONObject)usersPortion.get(i)).toJSONString()));
+							}
+
+							// avoid contention on a single retrieved value but add them all at the end
+							synchronized (toReturn){
+								toReturn.addAll(localList);
+							}
+						}
+					});
 				}
+
+				// wait threads to finish
+				executor.shutdown();
+				executor.awaitTermination(2, TimeUnit.MINUTES);
+
 			}else
 				return null;
 
@@ -513,42 +518,87 @@ public class LiferayWSUserManager implements UserManager{
 	}
 
 	@Override
-	public List<GCubeUser> listUsersByGroupAndRole(long groupId, long roleId)
+	public List<GCubeUser> listUsersByGroupAndRole(final long groupId, final long roleId)
 			throws UserManagementSystemException, RoleRetrievalFault,
 			GroupRetrievalFault, UserRetrievalFault {
 
-		List<GCubeUser> toReturn = listUsersByGroup(groupId);
+		final List<GCubeUser> toReturn = new ArrayList<GCubeUser>(0);
+
 		try {
 
+			final List<GCubeUser> usersInGroup = listUsersByGroup(groupId);
+			logger.debug("Number of users is " + usersInGroup.size());
 
-			Iterator<GCubeUser> iterator = toReturn.iterator();
-			while (iterator.hasNext()) {
-				GCubeUser gCubeUser = (GCubeUser) iterator.next();
-				long userId = gCubeUser.getUserId();
+			if(usersInGroup == null || usersInGroup.isEmpty())
+				return toReturn;
 
-				String userRoles = 
-						executeHTTPGETRequest(API_BASE_URL + GET_ROLES_IN_GROUP_BY_USER.replace("$GROUP_ID", String.valueOf(groupId)).replace("$USER_ID", String.valueOf(userId)), 
-								credsProvider, localContext, target);
+			// use executors to speed up this process
+			int numberOfThreads = (int) Math.ceil((double)usersInGroup.size()/(double)USERS_EXECUTOR_FACTOR);
+			logger.trace("Number of concurrent threads is going to be " + numberOfThreads);
 
-				boolean userHasRole = false;
-				JSONParser parser = new JSONParser();
-				JSONArray array = (JSONArray)parser.parse(userRoles);
-				for (int i = 0; i < array.size(); i++) {
-					try {
-						GCubeRole role = LiferayWSRoleManager.mapLRRole(array.get(i).toString());
-						if(role.getRoleId() == roleId){
-							userHasRole = true;
-							break;
+			// let do the job by some threads
+			ExecutorService executor = Executors.newFixedThreadPool(numberOfThreads);
+			int start = 0;
+			int offset = USERS_EXECUTOR_FACTOR;
+
+			for(int i = 0; i < numberOfThreads; i++){
+				start = USERS_EXECUTOR_FACTOR * i;
+				offset = (start + offset) > usersInGroup.size() ? usersInGroup.size() - start: offset;
+				logger.trace("Start = " + start + ", offset=" + offset);
+				final List<GCubeUser> usersPortion = usersInGroup.subList(start, start + offset);
+
+				executor.submit(new Runnable() {
+
+					@Override
+					public void run() {
+						logger.debug("Thread is " + Thread.currentThread().getName());
+						ArrayList<GCubeUser> localUsersHavingRole = new ArrayList<GCubeUser>(0);
+
+						for(GCubeUser user: usersPortion){
+							long userId = user.getUserId();
+							String userRoles = 
+									HttpUtils.executeHTTPGETRequest(API_BASE_URL + GET_ROLES_IN_GROUP_BY_USER.replace("$GROUP_ID", String.valueOf(groupId)).replace("$USER_ID", String.valueOf(userId)), 
+											credsProvider, localContext, target);
+
+							boolean userHasRole = false;
+							JSONParser parser = new JSONParser();
+							JSONArray array;
+							try {
+								array = (JSONArray)parser.parse(userRoles);
+
+								for (int i = 0; i < array.size(); i++) {
+									try {
+										GCubeRole role = LiferayWSRoleManager.mapLRRole(array.get(i).toString());
+										if(role.getRoleId() == roleId){
+											userHasRole = true;
+											break;
+										}
+									} catch (PortalException | SystemException | ParseException e) {
+										logger.warn("Failed to retrieve a role for user " + user.getUsername());
+									}
+								}
+
+								// add if he/she has
+								if(userHasRole)
+									localUsersHavingRole.add(user);
+
+							} catch (ParseException e1) {
+								logger.warn("Failed to parse role for user " + user.getUsername());
+							}
 						}
-					} catch (PortalException | SystemException | ParseException e) {
-						logger.warn("Failed to retrieve a role for user " + gCubeUser.getUsername());
-					}
-				}
 
-				if(!userHasRole)
-					iterator.remove();
+						// avoid contention on a single retrieved value but add them all at the end
+						synchronized (toReturn) {
+							toReturn.addAll(localUsersHavingRole);
+						}
+					}
+				});
 			}
 
+			// wait threads to finish
+			executor.shutdown();
+			executor.awaitTermination(2, TimeUnit.MINUTES);
+
 		} catch (Exception e) {
 			logger.warn("Failed to retrieve users that have role whit id " + roleId + " in group with id " + groupId);
 			return null;
@@ -647,7 +697,7 @@ public class LiferayWSUserManager implements UserManager{
 		String toReturn = null;
 		try{
 			String jsonCustomField = 
-					executeHTTPGETRequest(API_BASE_URL + GET_USER_CUSTOM_FIELD_BY_KEY.replace("$COMPANY_ID", String.valueOf(companyId)).replace("$CUSTOM_FIELD_KEY", attributeKey).replace("$USER_ID", String.valueOf(userId)), 
+					HttpUtils.executeHTTPGETRequest(API_BASE_URL + GET_USER_CUSTOM_FIELD_BY_KEY.replace("$COMPANY_ID", String.valueOf(companyId)).replace("$CUSTOM_FIELD_KEY", attributeKey).replace("$USER_ID", String.valueOf(userId)), 
 							credsProvider, localContext, target);
 			if(jsonCustomField != null){
 				logger.debug("Trying to parse custom field in json object");
@@ -676,7 +726,7 @@ public class LiferayWSUserManager implements UserManager{
 		List<GCubeUser> toReturn = null;
 
 		try{
-			String listIds = executeHTTPGETRequest(API_BASE_URL + GET_IDS_USERS_HAVING_GLOBAL_ROLE.replace("$ROLE_ID", String.valueOf(roleId)), 
+			String listIds = HttpUtils.executeHTTPGETRequest(API_BASE_URL + GET_IDS_USERS_HAVING_GLOBAL_ROLE.replace("$ROLE_ID", String.valueOf(roleId)), 
 					credsProvider, localContext, target);
 
 			if(listIds != null){
@@ -711,7 +761,7 @@ public class LiferayWSUserManager implements UserManager{
 
 		try{
 
-			String listIds = executeHTTPGETRequest(API_BASE_URL + GET_USER_ID_BY_GROUP.replace("$GROUP_ID", String.valueOf(groupId)), 
+			String listIds = HttpUtils.executeHTTPGETRequest(API_BASE_URL + GET_USER_ID_BY_GROUP.replace("$GROUP_ID", String.valueOf(groupId)), 
 					credsProvider, localContext, target);
 
 			if(listIds != null){
